/*
 *  JVending
 *  Copyright (C) 2004  Shane Isbell
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package javax.provisioning;

import java.util.Set;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.ServletContext;

/**
 * Interface defining the services provided by the Provisioning Framework. There is only one instance of
 * ProvisioningContext. It is created when the framework is initialized, and stored as a ServletContext attribute
 * with the name javax.provisioning.ProvisioningContext.
 */

public interface ProvisioningContext {

    public FulfillmentTask createFulfillmentTask(BundleDescriptor bundleDescriptor,
        DeliveryContext deliveryContext) throws ProvisioningException;
 
    public FulfillmentTask createFulfillmentTask(BundleDescriptor bundleDescriptor,
        DeliveryContext deliveryContext, String adapterName)throws ProvisioningException;

    public FulfillmentTask createFulfillmentTask(BundleDescriptor bundleDescriptor,
        DeliveryContext deliveryContext, String adapterName, long duration) throws ProvisioningException;

    public List getAdapterInfos();

    public List getAdapterInfos(BundleDescriptor bundleDescriptor, 
        String deviceIndentifier);  

    public BundleRepository getBundleRepository();
   
    public DeliveryContext getDeliveryContext(HttpServletRequest request);

    public Capabilities getDeviceTypeCapabilities(String indentifier);

    public Set getDeviceTypeIdentifiers();

    public ServletContext getServletContext();  
 

}