package org.jvending.provisioning.impl;

import junit.framework.TestCase;

import javax.provisioning.ProvisioningContext;
import javax.provisioning.DeliveryContext;
import javax.provisioning.Capabilities;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.easymock.MockControl;

import org.jvending.registry.RepositoryRegistry;
import org.jvending.provisioning.config.DeviceRepository;

import java.io.IOException;
import java.util.StringTokenizer;
import java.util.Set;
import java.util.List;

public class ProvisioningContextImplTest extends TestCase {

    private MockControl controlHttpServlet;

    private MockControl controlHttpSession;

    private HttpServletRequest httpServletRequest;

    private HttpSession httpSession;

    private static String baseDir = System.getProperty("basedir");

    private ProvisioningContext provisioningContext;

    public void setUp() {
        RepositoryRegistry repositoryRegistry = RepositoryRegistry.Factory.create();
        try {
            repositoryRegistry.loadFromFile(baseDir + "/src/test/resources/registry-config.xml");
        } catch (IOException e) {
            e.printStackTrace();
            fail("Could not load the registry-config file");
        }
        DeviceRepository deviceRepository = (DeviceRepository) repositoryRegistry.find("devices");
        provisioningContext = (ProvisioningContext) new ProvisioningContextImpl(deviceRepository, null, null, null);

        controlHttpServlet = MockControl.createControl(HttpServletRequest.class);
        controlHttpSession = MockControl.createControl(HttpSession.class);

        httpServletRequest = (HttpServletRequest) controlHttpServlet.getMock();
        httpSession = (HttpSession) controlHttpSession.getMock();
    }

    public void testDeviceTypeCapabilities() {
        Set ids = provisioningContext.getDeviceTypeIdentifiers();

        try {
            ids.add("Another");
            fail("The DeviceType Id set should be unmodifiable");
        } catch (Exception e) { }

        assertEquals("Incorrect number of device types", ids.size(), 9);
        assertTrue("Could not find device type: Value = Mozilla", ids.contains("Mozilla"));
        assertFalse("Contains bogus device type", ids.contains("Bogus"));
        assertTrue("Could not find device type: Value = Nokia6600", ids.contains("Nokia6600"));
    }

    public void testGetDeliveryContext() {
        httpServletRequest.getSession(false);
        controlHttpServlet.setReturnValue(httpSession);

        //Set up header names
        StringTokenizer tokenizer = new StringTokenizer("user-agent,MSISDN", ",");
        httpServletRequest.getHeaderNames();
        controlHttpServlet.setReturnValue(tokenizer);

        httpServletRequest.getHeader("user-agent");
        controlHttpServlet.setReturnValue("MOT-V300");

        httpServletRequest.getHeader("MSISDN");
        controlHttpServlet.setReturnValue("4251112222");

        controlHttpServlet.replay();
        controlHttpSession.replay();

        DeliveryContext deliveryContext = provisioningContext.getDeliveryContext(httpServletRequest);
        assertNotNull("Delivery Context is Null", deliveryContext);

        Capabilities capabilities = deliveryContext.getDeviceCapabilities();
        Set capabilityNames = capabilities.getCapabilityNames();
        assertEquals("Wrong number of capabilities for the MOT-V300", capabilityNames.size(), 3);
        assertTrue("Does not contain SoftwarePlatform.JavaPlatform", capabilityNames.contains("SoftwarePlatform.JavaPlatform"));
        assertTrue("Does not contain HardwarePlatform.BitsPerPixel", capabilityNames.contains("HardwarePlatform.BitsPerPixel"));

        List platforms = capabilities.getCapability("SoftwarePlatform.JavaPlatform");
        assertTrue("Incorrect platform value", platforms.contains("MIDP/2.0"));
        assertTrue("Incorrect platform value", platforms.contains("MIDP/1.0"));
        assertFalse("Incorrect platform value", platforms.contains("J2SE"));

        assertEquals("4251112222", deliveryContext.getNetworkID());
    }
}
