package org.jvending.provisioning.stocking.filter;

import junit.framework.TestCase;
import org.jvending.provisioning.stocking.par.ProvisioningArchive;
import org.jvending.provisioning.stocking.handler.StockingHandlerConfig;
import org.jvending.provisioning.stocking.StockingContext;
import org.jvending.provisioning.stocking.StockingComponent;
import org.jvending.provisioning.stocking.ProviderContext;
import org.easymock.MockControl;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.JAXBException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

public class BundleFilterTest extends TestCase {

    private static String baseDir = System.getProperty("basedir");

    private ProvisioningArchive archive;

    public void setUp() {
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(baseDir + "/src/test/resources/provisioning.xml");
        } catch (FileNotFoundException e) {
            fail("Could not find provisioning.xml file");
        }
        try {
            JAXBContext jaxbContext = JAXBContext.newInstance("org.jvending.provisioning.stocking.par");
            Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
            unmarshaller.setValidating(true);
            archive = (ProvisioningArchive) unmarshaller.unmarshal(fis);
        } catch (JAXBException e) {
            e.printStackTrace();
            fail("Parsing exception for provisioning descriptor");
        }
    }

    public void testBundleRemoval() {

        MockControl controlStockingContext = MockControl.createControl(StockingContext.class);
        StockingContext stockingContext = (StockingContext) controlStockingContext.getMock();

        stockingContext.getStockingComponent();
        controlStockingContext.setReturnValue(new StockingComponentImpl());

        MockControl controlStockingHandlerConfig = MockControl.createControl(StockingHandlerConfig.class);
        StockingHandlerConfig stockingHandlerConfig = (StockingHandlerConfig) controlStockingHandlerConfig.getMock();

        stockingHandlerConfig.getStockingContext();
        controlStockingHandlerConfig.setReturnValue(stockingContext);

        MockControl controlFilterTask = MockControl.createControl(FilterTask.class);
        FilterTask filterTask = (FilterTask) controlFilterTask.getMock();

        filterTask.getProvisioningArchive();
        controlFilterTask.setReturnValue(archive);

        filterTask.getStockingHandlerConfig();
        controlFilterTask.setReturnValue(stockingHandlerConfig);

        filterTask.getContent();
        controlFilterTask.setReturnValue(null);

        filterTask.getProviderContext();
        controlFilterTask.setReturnValue(new ProviderContextImpl());

        filterTask.getFilterID();
        controlFilterTask.setReturnValue("78932-DSJK");

        controlStockingContext.replay();
        controlStockingHandlerConfig.replay();
        controlFilterTask.replay();

        assertEquals("Do not have the expected number of bundles for input", 17, archive.getClientBundle().size());
        MockBundleFilter filter = new MockBundleFilter();
        filter.doFilter(filterTask);

        assertEquals("Did not remove bundle", 16, archive.getClientBundle().size());

    }

    private static class StockingComponentImpl extends StockingComponent {

    }

    private static class ProviderContextImpl implements ProviderContext {

        public String getUser() {
            return "test-user";
        }
    }



}
